
#include <stdio.h>
#include <wiringPi.h>




#include <sys/select.h> 
/* According to earlier standards */
#include <sys/time.h> 
#include <sys/types.h> 
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h> 
#include <time.h>
#include <stdarg.h>
#include <signal.h>
#include <errno.h>


#include <stdlib.h>
#include <string.h>
#include <iostream>


using namespace std;


#include "myserial.h"
#include "main.h"


// http://www.raspberry-projects.com/pi/programming-in-c/pipes/named-pipes-fifos

#define OUR_INPUT_FIFO_NAME "/tmp/stepperin"
#define OUR_OUTPUT_FIFO_NAME "/tmp/stepperout"

int our_input_fifo_filestream = -1;
int our_output_fifo_filestream = -1;

int makepipe(void)
{
 int result;

 umask(0);

 pr("Making FIFO...\n");
 result = mkfifo(OUR_INPUT_FIFO_NAME, 0777);		//(This will fail if the fifo already exists in the system from the app previously running, this is fine)
 if(result == 0)
 {
  //FIFO CREATED
  pr("New FIFO created: %s\n", OUR_INPUT_FIFO_NAME);
 }
 else
 {
  pr("FIFO already exists: %s\n", OUR_INPUT_FIFO_NAME);
 }



 result = mkfifo(OUR_OUTPUT_FIFO_NAME, 0777);		//(This will fail if the fifo already exists in the system from the app previously running, this is fine)
 if(result == 0)
 {
  //FIFO CREATED
  pr("New FIFO created: %s\n", OUR_OUTPUT_FIFO_NAME);
 }
 else
 {
  pr("FIFO already exists: %s\n", OUR_OUTPUT_FIFO_NAME);
 }


 pr("Process %d opening FIFO %s\n", getpid(), OUR_INPUT_FIFO_NAME);
 our_input_fifo_filestream = open(OUR_INPUT_FIFO_NAME, (O_RDONLY | O_NONBLOCK));
 if (our_input_fifo_filestream != -1) pr("Opened FIFO: %i\n", our_input_fifo_filestream);
 else                                 pr("error %d\n",errno);



 pr("Process %d opening FIFO %s\n", getpid(), OUR_OUTPUT_FIFO_NAME);

 our_output_fifo_filestream = open(OUR_OUTPUT_FIFO_NAME, (O_RDWR  | O_NONBLOCK));
 if(our_output_fifo_filestream != -1) pr("Opened FIFO: %i\n", our_output_fifo_filestream);
 else                                 pr("error %d\n",errno);


 return(0);
}





// http://cc.byexamples.com/2007/04/08/non-blocking-user-input-in-loop-without-ncurses/

int kbhit()
{
 struct timeval tv;
 fd_set fds;
 tv.tv_sec = 0;
 tv.tv_usec = 0;
 FD_ZERO(&fds);
 FD_SET(STDIN_FILENO, &fds); //STDIN_FILENO is 0
 select(STDIN_FILENO+1, &fds, NULL, NULL, &tv);
 return FD_ISSET(STDIN_FILENO, &fds);
}



#define MAXSERIAL 256

char serialbuffer[MAXSERIAL]=
{
 0

};


unsigned int serialnext=0;

int serialavailable=0;

int pipemode=0;
int debugmode=0;


void pr(const char *fmt, ... )
{
 if(!debugmode) return;
  
 char buf[256];
 va_list args;
 va_start (args, fmt );
 vsnprintf(buf,256, fmt, args);
 va_end (args);
 printf(buf);

}



void checkserial(void)
{
 int n;

 if(pipemode &&  our_input_fifo_filestream!=-1)
 {
  if(serialbuffer[serialnext]==0)
  {
   serialavailable=0;
   n=read(our_input_fifo_filestream,serialbuffer,sizeof(serialbuffer));
   if(n>0)
   {
    serialnext=0;
    serialavailable=1;
    serialbuffer[n]=0;
   }
  }
  else
  {
   serialavailable=1;
  }
 }
 else
 {
  if(serialbuffer[serialnext]==0)
  {
   serialavailable=0;
   if(kbhit())
   {
    if(fgets(serialbuffer,sizeof(serialbuffer),stdin))
    {
     serialnext=0;
     serialavailable=1;
    }
   }
  }
  else
  {
   serialavailable=1;
  }
 }
}




MySerial::MySerial()
{


}

      // Constructor
void MySerial::print(string s,int format)
{
 char buffer[256];

 if(pipemode &&  our_output_fifo_filestream!=-1)
 {
  sprintf(buffer,"%s",s.c_str());
  write(our_output_fifo_filestream,buffer,strlen(buffer));
 }

 if(debugmode || !pipemode)
 {
  printf("%s",s.c_str());
 }
}

void MySerial::println(string s,int format)
{
 char buffer[256];

 if(pipemode &&  our_output_fifo_filestream!=-1)
 {
  sprintf(buffer,"%s\n",s.c_str());
  write(our_output_fifo_filestream,buffer,strlen(buffer));
 }

 if(debugmode || !pipemode)
 {
  printf("%s\n",s.c_str());
 }
}


void MySerial::print(long s,int format)
{
 char buffer[256];

 if(pipemode &&  our_output_fifo_filestream!=-1)
 {
  sprintf(buffer,"%ld",s);
  write(our_output_fifo_filestream,buffer,strlen(buffer));
 }

 if(debugmode || !pipemode)
 {
  printf("%ld",s);
 }
}

void MySerial::println(long s,int format)
{
 char buffer[256];

 if(pipemode &&  our_output_fifo_filestream!=-1)
 {
  sprintf(buffer,"%ld\n",s);
  write(our_output_fifo_filestream,buffer,strlen(buffer));
 }

 if(debugmode || !pipemode)
 {
  printf("%ld\n",s);
 }
}


void MySerial::begin(int baud)
{



}


int MySerial::available(void)
{
 checkserial();
 return(serialavailable);
}


int MySerial::read(void)
{
 checkserial();
 if(serialavailable)
 {
  return(serialbuffer[serialnext++]);
 }
}



extern void setup(void);
extern void loop(void);
extern void setdown(void);



// http://www.thegeekstuff.com/2012/03/catch-signals-sample-c-code/

void sig_handler(int signo)
{

// setdown();
 exit(1);
}




int main (int argc,char * argv[])
{
 int i;

 for(i=0;i<argc;i++)
 {
  if(!strcmp(argv[i],"pipe")) pipemode=1;
  if(!strcmp(argv[i],"debug")) debugmode=1;
 }


 wiringPiSetup () ;

 if(pipemode) makepipe();

 setup();
 atexit(setdown);
 signal(SIGINT,sig_handler);
 signal(SIGSEGV,sig_handler);


 while(1)
 {
  loop();
  usleep(0);
 }



 return 0 ;
}
